/*
 * Copyright (c) 1998 by Tsuyoshi Iguchi (zinnia@jan.ne.jp)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */
/*
 * pcmtest -- test PCM driver
 */
#include <unistd.h>
#include <machine/soundcard.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>

typedef struct{
  int channel;
  int rate;
  int bits;
} WAV;

WAV orig, act;
int dsp;

int rate_tbl[] = {48000, 44100, 37800, 33075, 32000, 27429, 22050,
		  18900, 16000, 11025, 9600, 8000, 6615, 5512, 0};
int rate_n;
int* rate_check;  /* 0 = Ok, >0 = NG (result rate), <0 = error */
char* dspdev = "/dev/dsp";

int main(int argc, char* argv[])
{
  int i; int bits, channel;
  int ok;
  int readable = 0;

  if(argc > 1){ dspdev = argv[1]; }

  for(i = 0; rate_tbl[i] != 0; i++);
  rate_n = i;
  rate_check = (int*)calloc(rate_n, sizeof(int));
  if(rate_check == NULL){ 
    fprintf(stderr, "Memory allocation error.\n");
    exit(1);
  }
  dsp = open(dspdev, O_RDWR);
  if(dsp < 0){
    fprintf(stderr, "%s: open error for O_RDWR, trying to O_WRONLY...\n", dspdev);
    dsp = open(dspdev, O_WRONLY);
    if(dsp < 0){
      fprintf(stderr, "%s: open error for O_WRONLY, giving up.\n", dspdev);
      exit(1);
    }
    readable = 0;
  } else {
    readable = 1;
  }

  printf("**** Play ****\n");
  bits = 0;
  printf("  8 bit I/O: ");
  orig.bits = act.bits = 8;
  if(ioctl(dsp, SOUND_PCM_WRITE_BITS, &act.bits) == -1){
    printf("NG (%s)\n", strerror(errno));
  } else {
    if(orig.bits == act.bits){
      printf("Ok \n"); bits = act.bits;
    } else {
      printf("NG (actually set to %d)\n", act.bits);
    }
  }

  printf(" 16 bit I/O: ");
  orig.bits = act.bits = 16;
  if(ioctl(dsp, SOUND_PCM_WRITE_BITS, &act.bits) == -1){
    printf("NG (%s)\n", strerror(errno));
  } else {
    if(orig.bits == act.bits){
      printf("Ok \n"); bits = act.bits;
    } else {
      printf("NG (actually set to %d)\n", act.bits);
    }
  }
  if(bits == 0){
    fprintf(stderr, "FATAL: I can't find any sampling bit available.\n");
    exit(1);
  }

  printf("Monoral I/O: ");
  orig.channel = act.channel = 1;   /* monoral */
  if(ioctl(dsp, SOUND_PCM_WRITE_CHANNELS, &act.channel) == -1){
    printf("NG (%s)\n", strerror(errno));
  } else {
    if(orig.channel == act.channel){
      printf("Ok \n"); channel = act.channel;
    } else {
      printf("NG (actually set to %d)\n", act.channel);
    }
  }

  printf(" Stereo I/O: ");
  orig.channel = act.channel = 2;   /* monoral */
  if(ioctl(dsp, SOUND_PCM_WRITE_CHANNELS, &act.channel) == -1){
    printf("NG (%s)\n", strerror(errno));
  } else {
    if(orig.channel == act.channel){
      printf("Ok \n"); channel = act.channel;
    } else {
      printf("NG (actually set to %d)\n", act.channel);
    }
  }

  if(channel == 0){
    fprintf(stderr, "FATAL: I can't find any channel available.\n");
    exit(1);
  }
  printf("Sample rate:\n");
  for(i = 0; i < rate_n; i++){
    orig.rate = act.rate = rate_tbl[i];
    orig.bits = bits;
    orig.channel = channel;
    if(ioctl(dsp, SOUND_PCM_WRITE_RATE, &act.rate) < 0){
      printf("(%d: %s)\n", orig.rate, strerror(errno));
    } else {
      if(act.rate == orig.rate){
	printf(" %d\n", act.rate);
      } else {
	printf("(%d -> %d)\n", orig.rate, act.rate);
      }
    }
  }
  printf("\n");

  if(readable){
    printf("**** Record ****\n");
    bits = 0;
    printf("  8 bit I/O: ");
    orig.bits = act.bits = 8;
    if(ioctl(dsp, SOUND_PCM_READ_BITS, &act.bits) == -1){
      printf("NG (%s)\n", strerror(errno));
    } else {
      if(orig.bits == act.bits){
	printf("Ok \n"); bits = act.bits;
      } else {
	printf("NG (actually set to %d)\n", act.bits);
      }
    }
    
    printf(" 16 bit I/O: ");
    orig.bits = act.bits = 16;
    if(ioctl(dsp, SOUND_PCM_READ_BITS, &act.bits) == -1){
      printf("NG (%s)\n", strerror(errno));
    } else {
      if(orig.bits == act.bits){
	printf("Ok \n"); bits = act.bits;
      } else {
	printf("NG (actually set to %d)\n", act.bits);
      }
    }
    if(bits == 0){
      fprintf(stderr, "FATAL: I can't find any sampling bit available.\n");
      exit(1);
    }
    
    printf("Monoral I/O: ");
    orig.channel = act.channel = 1;   /* monoral */
    if(ioctl(dsp, SOUND_PCM_READ_CHANNELS, &act.channel) == -1){
      printf("NG (%s)\n", strerror(errno));
    } else {
      if(orig.channel == act.channel){
	printf("Ok \n"); channel = act.channel;
      } else {
	printf("NG (actually set to %d)\n", act.channel);
      }
    }
    
    printf(" Stereo I/O: ");
    orig.channel = act.channel = 2;   /* monoral */
    if(ioctl(dsp, SOUND_PCM_READ_CHANNELS, &act.channel) == -1){
      printf("NG (%s)\n", strerror(errno));
    } else {
      if(orig.channel == act.channel){
	printf("Ok \n"); channel = act.channel;
      } else {
	printf("NG (actually set to %d)\n", act.channel);
      }
    }
    
    if(channel == 0){
      fprintf(stderr, "FATAL: I can't find any channel available.\n");
      exit(1);
    }
    printf("Sample rate:\n");
    for(i = 0; i < rate_n; i++){
      orig.rate = act.rate = rate_tbl[i];
      orig.bits = bits;
      orig.channel = channel;
      if(ioctl(dsp, SOUND_PCM_READ_RATE, &act.rate) < 0){
	printf("(%d: %s)\n", orig.rate, strerror(errno));
      } else {
	if(act.rate == orig.rate){
	  printf(" %d\n", act.rate);
	} else {
	  printf("(%d -> %d)\n", orig.rate, act.rate);
	}
      }
    }
  }
  printf("\n");
}
